/* Dynamic reports - Free Java reporting library for creating reports dynamically
 *
 * (C) Copyright 2010 Ricardo Mariaca
 *
 * http://dynamicreports.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by 
 * the Free Software Foundation; either version 3 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA. 
 */
package net.sf.dynamicreports.report.builder.subtotal;

import net.sf.dynamicreports.report.builder.DynamicReports;
import net.sf.dynamicreports.report.builder.FieldBuilder;
import net.sf.dynamicreports.report.builder.column.ColumnBuilder;
import net.sf.dynamicreports.report.constant.Calculation;
import net.sf.dynamicreports.report.definition.expression.DRIComplexExpression;
import net.sf.dynamicreports.report.definition.expression.DRISimpleExpression;

import org.apache.commons.lang.Validate;

/**
 * @author Ricardo Mariaca (dynamicreports@gmail.com)
 */
@SuppressWarnings("ucd")
public class Subtotals {

	//calculation
	public static <T> AggregationSubtotalBuilder<T> aggregate(ColumnBuilder<?, ?> subtotalColumn, Calculation calculation) {
		Validate.notNull(subtotalColumn, "subtotalColumn must not be null");
		return new AggregationSubtotalBuilder<T>(subtotalColumn, calculation);
	}

	public static <T> AggregationSubtotalBuilder<T> aggregate(String fieldName, Class<?> valueClass, ColumnBuilder<?, ?> showInColumn, Calculation calculation) {
		return aggregate(DynamicReports.field(fieldName, valueClass), showInColumn, calculation);
	}

	public static <T> AggregationSubtotalBuilder<T> aggregate(FieldBuilder<?> field, ColumnBuilder<?, ?> showInColumn, Calculation calculation) {
		Validate.notNull(showInColumn, "showInColumn must not be null");
		return new AggregationSubtotalBuilder<T>(field, showInColumn, calculation);
	}

	public static <T> AggregationSubtotalBuilder<T> aggregate(DRISimpleExpression<?> expression, ColumnBuilder<?, ?> showInColumn, Calculation calculation) {
		Validate.notNull(showInColumn, "showInColumn must not be null");
		return new AggregationSubtotalBuilder<T>(expression, showInColumn, calculation);
	}
	
	//sum
	public static <T extends Number> AggregationSubtotalBuilder<T> sum(ColumnBuilder<?, T> subtotalColumn) {
		return aggregate(subtotalColumn, Calculation.SUM);
	}

	public static <T extends Number> AggregationSubtotalBuilder<T> sum(String fieldName, Class<T> valueClass, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(fieldName, valueClass, showInColumn, Calculation.SUM);
	}

	public static <T extends Number> AggregationSubtotalBuilder<T> sum(FieldBuilder<T> field, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(field, showInColumn, Calculation.SUM);
	}

	public static <T extends Number> AggregationSubtotalBuilder<T> sum(DRISimpleExpression<T> expression, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(expression, showInColumn, Calculation.SUM);
	}
	
	//average
	public static <T extends Number> AggregationSubtotalBuilder<Number> avg(ColumnBuilder<?, T> subtotalColumn) {
		return aggregate(subtotalColumn, Calculation.AVERAGE);
	}

	public static <T extends Number> AggregationSubtotalBuilder<Number> avg(String fieldName, Class<T> valueClass, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(fieldName, valueClass, showInColumn, Calculation.AVERAGE);
	}

	public static <T extends Number> AggregationSubtotalBuilder<Number> avg(FieldBuilder<T> field, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(field, showInColumn, Calculation.AVERAGE);
	}

	public static <T extends Number> AggregationSubtotalBuilder<Number> avg(DRISimpleExpression<T> expression, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(expression, showInColumn, Calculation.AVERAGE);
	}
	
	//count
	public static AggregationSubtotalBuilder<Long> count(ColumnBuilder<?, ?> subtotalColumn) {
		return aggregate(subtotalColumn, Calculation.COUNT);
	}

	public static AggregationSubtotalBuilder<Long> count(String fieldName, Class<?> valueClass, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(fieldName, valueClass, showInColumn, Calculation.COUNT);
	}

	public static AggregationSubtotalBuilder<Long> count(FieldBuilder<?> field, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(field, showInColumn, Calculation.COUNT);
	}

	public static AggregationSubtotalBuilder<Long> count(DRISimpleExpression<?> expression, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(expression, showInColumn, Calculation.COUNT);
	}
	
	//distinct count
	public static AggregationSubtotalBuilder<Long> distinctCount(ColumnBuilder<?, ?> subtotalColumn) {
		return aggregate(subtotalColumn, Calculation.DISTINCT_COUNT);
	}

	public static AggregationSubtotalBuilder<Long> distinctCount(String fieldName, Class<?> valueClass, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(fieldName, valueClass, showInColumn, Calculation.DISTINCT_COUNT);
	}

	public static AggregationSubtotalBuilder<Long> distinctCount(FieldBuilder<?> field, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(field, showInColumn, Calculation.DISTINCT_COUNT);
	}

	public static AggregationSubtotalBuilder<Long> distinctCount(DRISimpleExpression<?> expression, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(expression, showInColumn, Calculation.DISTINCT_COUNT);
	}
	
	//first
	public static <T> AggregationSubtotalBuilder<T> first(ColumnBuilder<?, T> subtotalColumn) {
		return aggregate(subtotalColumn, Calculation.FIRST);
	}

	public static <T> AggregationSubtotalBuilder<T> first(String fieldName, Class<T> valueClass, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(fieldName, valueClass, showInColumn, Calculation.FIRST);
	}

	public static <T> AggregationSubtotalBuilder<T> first(FieldBuilder<T> field, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(field, showInColumn, Calculation.FIRST);
	}

	public static <T> AggregationSubtotalBuilder<T> first(DRISimpleExpression<T> expression, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(expression, showInColumn, Calculation.FIRST);
	}
	
	//highest
	public static <T> AggregationSubtotalBuilder<T> max(ColumnBuilder<?, T> subtotalColumn) {
		return aggregate(subtotalColumn, Calculation.HIGHEST);
	}

	public static <T> AggregationSubtotalBuilder<T> max(String fieldName, Class<T> valueClass, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(fieldName, valueClass, showInColumn, Calculation.HIGHEST);
	}

	public static <T> AggregationSubtotalBuilder<T> max(FieldBuilder<T> field, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(field, showInColumn, Calculation.HIGHEST);
	}

	public static <T> AggregationSubtotalBuilder<T> max(DRISimpleExpression<T> expression, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(expression, showInColumn, Calculation.HIGHEST);
	}
	
	//lowest
	public static <T> AggregationSubtotalBuilder<T> min(ColumnBuilder<?, T> subtotalColumn) {
		return aggregate(subtotalColumn, Calculation.LOWEST);
	}

	public static <T> AggregationSubtotalBuilder<T> min(String fieldName, Class<T> valueClass, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(fieldName, valueClass, showInColumn, Calculation.LOWEST);
	}

	public static <T> AggregationSubtotalBuilder<T> min(FieldBuilder<T> field, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(field, showInColumn, Calculation.LOWEST);
	}

	public static <T> AggregationSubtotalBuilder<T> min(DRISimpleExpression<T> expression, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(expression, showInColumn, Calculation.LOWEST);
	}
	
	//standard deviation
	public static <T extends Number> AggregationSubtotalBuilder<Number> stdDev(ColumnBuilder<?, T> subtotalColumn) {
		return aggregate(subtotalColumn, Calculation.STANDARD_DEVIATION);
	}

	public static <T extends Number> AggregationSubtotalBuilder<Number> stdDev(String fieldName, Class<T> valueClass, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(fieldName, valueClass, showInColumn, Calculation.STANDARD_DEVIATION);
	}

	public static <T extends Number> AggregationSubtotalBuilder<Number> stdDev(FieldBuilder<T> field, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(field, showInColumn, Calculation.STANDARD_DEVIATION);
	}

	public static <T extends Number> AggregationSubtotalBuilder<Number> stdDev(DRISimpleExpression<T> expression, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(expression, showInColumn, Calculation.STANDARD_DEVIATION);
	}
	
	//variance
	public static <T extends Number> AggregationSubtotalBuilder<Number> var(ColumnBuilder<?, T> subtotalColumn) {
		return aggregate(subtotalColumn, Calculation.VARIANCE);
	}

	public static <T extends Number> AggregationSubtotalBuilder<Number> var(String fieldName, Class<T> valueClass, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(fieldName, valueClass, showInColumn, Calculation.VARIANCE);
	}

	public static <T extends Number> AggregationSubtotalBuilder<Number> var(FieldBuilder<T> field, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(field, showInColumn, Calculation.VARIANCE);
	}

	public static <T extends Number> AggregationSubtotalBuilder<Number> var(DRISimpleExpression<T> expression, ColumnBuilder<?, ?> showInColumn) {
		return aggregate(expression, showInColumn, Calculation.VARIANCE);
	}
	
	//custom
	public static <T> CustomSubtotalBuilder<T> customValue(DRISimpleExpression<T> expression, ColumnBuilder<?, ?> showInColumn) {
		Validate.notNull(showInColumn, "showInColumn must not be null");
		return new CustomSubtotalBuilder<T>(expression, showInColumn);	
	}

	public static <T> CustomSubtotalBuilder<T> customValue(DRIComplexExpression<T> expression, ColumnBuilder<?, ?> showInColumn) {
		Validate.notNull(showInColumn, "showInColumn must not be null");
		return new CustomSubtotalBuilder<T>(expression, showInColumn);	
	}
	
	//percentage
	public static PercentageSubtotalBuilder percentage(ColumnBuilder<?, ? extends Number> subtotalColumn) {
		Validate.notNull(subtotalColumn, "subtotalColumn must not be null");
		return new PercentageSubtotalBuilder(subtotalColumn);
	}
	
	public static PercentageSubtotalBuilder percentage(String fieldName, Class<? extends Number> valueClass, ColumnBuilder<?, ?> showInColumn) {
		return percentage(DynamicReports.field(fieldName, valueClass), showInColumn);
	}
	
	public static PercentageSubtotalBuilder percentage(FieldBuilder<? extends Number> field, ColumnBuilder<?, ?> showInColumn) {
		Validate.notNull(showInColumn, "showInColumn must not be null");
		return new PercentageSubtotalBuilder(field, showInColumn);
	}
	
	/*public static PercentageSubtotalBuilder percentage(DRISimpleExpression<? extends Number> expression, ColumnBuilder<?, ?> showInColumn) {
		Validate.notNull(showInColumn, "showInColumn must not be null");
		return new PercentageSubtotalBuilder(expression, showInColumn);
	}*/
}
